#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import fnmatch
import logging
from apiclient import errors
from command_base import DriveServiceCommand
from command_list import CommandList

lg = logging.getLogger("DRIVE.SHARE")
# lg.setLevel(logging.INFO)


class CommandShare(CommandList):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('share',
                                        help='command share help')
        cmdparser.add_argument('src', nargs='+',
                               help='patterns to list in google drive')

        cmdparser.add_argument('-u', '--unshare', action='store_true',
                               help='unshare files')

    def do_service_command(self):
        """share files
        """

        lg.debug("YMK in do_command")
        lg.debug(self.args)

        files = self.get_all_src_files(self.args.src, False)

        if len(files) == 0:
            sys.exit("No files matched in drive")

        sfiles = []
        for pidx in range(len(files)):
            perms = self.permission_list(files[pidx]['id'])
            perms = filter(lambda p: p['type'] == 'anyone', perms)
            shared = 'shared' if len(perms) > 0 else ''

            if 'webContentLink' in files[pidx]:
                link = files[pidx]['webContentLink']
            elif 'alternateLink' in files[pidx]:
                link = files[pidx]['alternateLink']
            else:
                link = files[pidx]['id']

            sfiles.append({'idx': pidx, 'title': files[pidx]['title'], 'id': files[pidx]['id'],
                           'shared': shared, 'link': link})
            self.info("%2d %s \n  (%s)  %s" % (pidx, files[pidx]['title'], shared, link))

        inpstr = self.choose_files(sfiles)
        allidxs = self.parse_input_string(inpstr, len(sfiles))
        for aidx in allidxs:
            if self.args.unshare is True:
                if sfiles[aidx]['shared'] == 'shared':
                    self.info("unsharing %s" % sfiles[aidx]['title'])
                    self.unshare_a_file(sfiles[aidx]['id'])
            else:
                if sfiles[aidx]['shared'] == '':
                    self.info("sharing %s" % sfiles[aidx]['title'])
                    self.share_a_file(sfiles[aidx]['id'])

# ## private methods ##
    def share_a_file(self, pfid, pvalue=None, ptype='anyone', prole='reader'):
        nperm = {
            'value': pvalue,
            'type': ptype,
            'role': prole
        }
        try:
            return self.service.permissions().insert(fileId=pfid, body=nperm).execute()
        except errors.HttpError, error:
            print 'An error occurred: %s' % error
            return None
        self.info("%s shared" % (pfid))

    def unshare_a_file(self, pfid, pvalue=None, ptype='anyone', prole='reader'):
        perms = self.permission_list(pfid)
        perms = filter(lambda p: p['type'] == 'anyone', perms)
        for aperm in perms:
            try:
                self.service.permissions().delete(
                    fileId=pfid, permissionId=aperm['id']).execute()
            except errors.HttpError, error:
                print 'An error occurred: %s' % error
                return None
        self.info("fid %s unshared" % (pfid))



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import fnmatch
import logging
from colorama import Fore, Style
from apiclient import errors
from command_base import DriveServiceCommand

lg = logging.getLogger("DRIVE.LIST")
# lg.setLevel(logging.INFO)


class CommandList(DriveServiceCommand):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('list',
                                        help='command list help')
        cmdparser.add_argument('src', nargs='+',
                               help='patterns to list in google drive')

    def do_service_command(self):
        """list files
        """

        lg.debug("YMK in do_command")
        lg.debug(self.args)

        files = self.get_all_src_files(self.args.src)

        if len(files) == 0:
            sys.exit("No files matched in drive")

        self.show_files_info(files)
        # for pidx in range(len(files)):
        #     if 'weViewLink' in files[pidx]:
        #         self.info("%d %s %s" % (pidx, files[pidx]['title'], files[pidx]['webViewLink']))
        #     elif 'webContentLink' in files[pidx]:
        #         self.info("%d %s %s" % (pidx, files[pidx]['title'], files[pidx]['webContentLink']))
        #     else:
        #         self.info("%d %s %s %s" % (pidx, files[pidx]['title'], files[pidx]['id'], files[pidx]['ownerNames'][0]))

# ## private methods ##
    def get_all_src_files(self, psrc, hidedir=False):
        allfiles = []
        for asrc in psrc:
            dirname = os.path.dirname(asrc)
            basename = os.path.basename(asrc)
            if basename == "":
                basename = '*'
            lg.debug("src dirname %s basename %s" % (dirname, basename))
            files = self.find_drive_files(dirname, basename, hidedir=hidedir)
            allfiles.extend(files)
        return allfiles

    def show_files_info(self, pfiles, pnum=False, plong=False):
        # if pnum is False:
        #     if plong is False:
        #         for apf in pfiles:
        #             self.info()

        if plong is False:
            for pidx in range(len(pfiles)):
                apf = pfiles[pidx]
                self.info_append(Fore.GREEN +
                                 "%s" % ("%2d " % pidx if pnum is True else "") +
                                 Style.RESET_ALL)
                self.info("%s" % (Fore.BLUE + apf['title'] + Style.RESET_ALL
                                  if apf['mimeType'] == 'application/vnd.google-apps.folder'
                                  else apf['title']))

    def choose_files(self, pfiles):
        self.info(Fore.GREEN +
                  "[a]= all, [0-%d]: number: " % (len(pfiles) - 1) +
                  Style.RESET_ALL)
        return raw_input().strip()



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

# __all__ = []

# import pkgutil
# import inspect

# for loader, name, is_pkg in pkgutil.walk_packages(__path__):
#     # print("YMK name {0}".format(name))
#     module = loader.find_module(name).load_module(name)
#
#     for name, value in inspect.getmembers(module):
#         if name.startswith('__'):
#             continue
#
#         globals()[name] = value
#         __all__.append(name)
# print("YMK __all__ {0}".format(__all__))



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import logging
import progressbar
from apiclient import errors
from apiclient.http import MediaFileUpload
from command_base import DriveServiceCommand

lg = logging.getLogger("DRIVE.PUSH")
# lg.setLevel(logging.INFO)


class CommandPush(DriveServiceCommand):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('push',
                                        help='command push help')
        # ### for query string composing ###
        cmdparser.add_argument('src', nargs='+',
                               help='source files')
        cmdparser.add_argument('dst', nargs=1,
                               help='desination')
        cmdparser.add_argument('-p', '--prarents', nargs=1,
                               help='no error if existing,'
                               'make parent directories as needed')

    def do_service_command(self):
        """push files
        """

        lg.debug("YMK in do_command")
        lg.debug(self.args)
        # sys.stderr.write("YMK STDERR\n")

        if len(self.args.src) > 1:
            if os.path.basename(self.args.dst[0]) != "":
                lg.error("Multiple source should put in a directory")
                sys.exit("Multiple source should put in a directory")
        else:
            title = os.path.basename(self.args.dst[0])

        parentid = self.find_dst_dir()
        if parentid is None:
            lg.error("Can't find directory %s in drive" % self.args.dst[0])
            sys.exit("Can't find directory %s in drive" % self.args.dst[0])

        # ## check src files exists in local
        for src in self.args.src:
            if not os.path.exists(src):
                lg.error("%s dosen't exists" % src)
                sys.exit("%s dosen't exists" % src)

        # ## TODO check files exist in drive
        # ## TODO to create or update

        for src in self.args.src:
            if title == "":
                title = os.path.basename(src)
            lg.debug("src %s title %s put in %s" % (src, title, parentid))
            afile = self.file_insert(src, title, parentid)
            if afile is None or not afile['title'] == title:
                lg.error("File %s push error", src)
            else:
                lg.debug("File %s pushed in drive as id %s" % (
                    afile['title'], afile['id']))

# ## private methods ##
    def file_insert(self, filename, title, parent_id):
        """Insert new file.

        Args:
            title: Title of the file to insert, including the extension.
            parent_id: Parent folder's ID.
            filename: Filename of the file to insert.
        Returns:
            Inserted file metadata if successful, None otherwise.
        """

        media_body = MediaFileUpload(filename, resumable=True)
        # ## TODO: add optioneal properties.
        # 'description': description,
        # 'mimeType': mime_type
        body = {
            'title': title
        }
        # Set the parent folder.
        if parent_id:
            body['parents'] = [{'id': parent_id}]

        try:
            req = self.service.files().insert(
                body=body,
                media_body=media_body)
            self.info("%s uploading ..." % title)
            pbar = progressbar.ProgressBar(
                widgets=[progressbar.Percentage(),
                         progressbar.Bar()],
                maxval=100).start()
            res = None
            while res is None:
                status, res = req.next_chunk()
                if status:
                    # lg.debug("%d%% uploaded", int(status.progress() * 100))
                    pbar.update(int(status.progress() * 100))
            return res
        except errors.HttpError, error:
            lg.error('An error occured: %s' % error)
            return None

    def find_dst_dir(self):
        dstdir = os.path.dirname(self.args.dst[0])
        return self.find_parent_id(dstdir)

# ## not used
    def get_children_dirs(self, parent="root"):
        query = "mimeType = 'application/vnd.google-apps.folder'"
        query += " and '%s' in parents" % parent
        # return self.children_list(parent, query)
        return self.file_list(query)

    def get_all_dirs(self):
        query = "mimeType = 'application/vnd.google-apps.folder'"
        return self.file_list(query)



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import logging
from apiclient import errors
from command_base import DriveServiceCommand

lg = logging.getLogger("DRIVE.SEARCH")
# lg.setLevel(logging.INFO)


class CommandSearch(DriveServiceCommand):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('search',
                                        help='command search help')
        cmdparser.add_argument('queries', nargs='?',
                               help='other query string for search files')
        cmdparser.add_argument('-m', '--max-results',
                               type=int, default=100,
                               help='maximum number of files to return')
        cmdparser.add_argument('-o', '--operator',
                               choices=['and', 'or'], default='and',
                               help='logical operator between query'
                               'strings')

        # ### for query string composing ###
        cmdparser.add_argument('-t', '--title', nargs='*',
                               help='title of the file')
        cmdparser.add_argument('-f', '--full-text', nargs='*',
                               help='full text of the file including'
                               'title, description, and content')

    def do_service_command(self):
        """list files
        """

        lg.debug("YMK in do_command")
        lg.debug(self.args)

        files = self.retrieve_files()
        for fl in files:
            print fl
            # print "%s, %s, %s" % (fl['title'], fl['id'], fl['mimeType'])

# ## private methods ##
    def retrieve_files(self, query=""):
        """Retrieve a list of File resources.

        Args:
          service: Drive API service instance.
        Returns:
          List of File resources.
        """
        result = []
        page_token = None
        while True:
            try:
                param = {}
                if query != "":
                    param['q'] = "title contains '%s'" % (query)
                if page_token:
                    param['pageToken'] = page_token
                files = self.service.files().list(**param).execute()

                result.extend(files['items'])
                page_token = files.get('nextPageToken')
                if not page_token:
                    break
            except errors.HttpError, error:
                print 'An error occurred: %s' % error
                break
        return result



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import logging
import webbrowser
import httplib2
from command_base import DriveServiceCommand

from apiclient import errors
from apiclient.discovery import build
from oauth2client.file import Storage
from oauth2client.client import AccessTokenRefreshError
from oauth2client.client import OAuth2WebServerFlow
# from oauth2client.tools import run

lg = logging.getLogger("DRIVE.INIT")
# lg.setLevel(logging.INFO)


class CommandInit(DriveServiceCommand):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        # ## python2.7 lack of aliases of add_parser in sub command.
        cmdparser = psub_par.add_parser('init',
                                        help='command init help')
        cmdparser.add_argument('queries', nargs='?',
                               help='other query string for search files')
        cmdparser.add_argument('-m', '--max-results',
                               type=int, default=100,
                               help='maximum number of files to return')
        cmdparser.add_argument('-o', '--operator',
                               choices=['and', 'or'], default='and',
                               help='logical operator between query'
                               'strings')

        # ## for query string composing ## #
        cmdparser.add_argument('-t', '--title', nargs='*',
                               help='title of the file')
        cmdparser.add_argument('-f', '--full-text', nargs='*',
                               help='full text of the file including'
                               'title, description, and content')

    def do_drive_command(self):
        lg.debug("YMK in do_command")
        lg.debug(self.args)
        lg.debug("YMK dump config api ")
        lg.debug(self.config.items('api'))

        flow = OAuth2WebServerFlow(client_id=self.config.get('api', 'client_id'),
                                   client_secret=self.config.get('api', 'client_secret'),
                                   scope=self.config.get('api', 'scope'),
                                   redirect_uri="urn:ietf:wg:oauth:2.0:oob")

        self.get_credentials()
        if self.credentials is None or self.credentials.invalid:
            # YMK: this flow no longer works

            # flow_info = flow.step1_get_device_and_user_codes()
            # webbrowser.open_new_tab(flow_info.verification_url)
            # print("Enter verrification code in url {0}: {1}".format(flow_info.verification_url, flow_info.user_code))
            # raw_input("Then press any key to continue...".format(flow_info.user_code))
            # self.credentials = flow.step2_exchange(device_flow_info=flow_info)

            auth_uri = flow.step1_get_authorize_url()
            webbrowser.open_new_tab(auth_uri)
            print("Please goto this link: [%s]".format(auth_uri))
            code = raw_input("Enter verrification code: ").strip()
            self.credentials = flow.step2_exchange(code)
            if self.credentials:
                filename = os.path.expanduser(self.config.get('api', 'storage'))
                dirname = os.path.dirname(filename)
                if not os.path.exists(dirname):
                    os.makedirs(dirname)
                self.storage.put(self.credentials)

        self.get_service()
        if self.service is not None:
            self.do_service_command()

    def do_service_command(self):
        self.print_about(self.service)

# ## private methods ##
    def print_about(self, service):
        """Print information about the user along with the Drive API settings.

        Args:
          service: Drive API service instance.
        """
        try:
            about = service.about().get().execute()

            print 'Current user name: %s' % about['name']
            print 'Root folder ID: %s' % about['rootFolderId']
            print 'Total quota (bytes): %s' % about['quotaBytesTotal']
            print 'Used quota (bytes): %s' % about['quotaBytesUsed']
        except errors.HttpError, error:
            print 'An error occurred: %s' % error



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import logging
import urllib2
import fnmatch
import colorama
import progressbar
from apiclient import errors
from command_list import CommandList

lg = logging.getLogger("DRIVE.TRASH")
# lg.setLevel(logging.INFO)


class CommandTrash(CommandList):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('trash',
                                        help='command trash help')
        cmdparser.add_argument('src', nargs='+',
                               help='google drive files')

    def do_service_command(self):
        """trash files
        """

        # lg.debug(self.args)
        pulls = self.get_all_src_files(self.args.src, hidedir=False)

        if len(pulls) == 0:
            sys.exit("No files matched in drive")
        self.info(colorama.Fore.RED +
                  "Would you like to trash these files ?" +
                  colorama.Style.RESET_ALL)
        self.show_files_info(pulls, pnum=True)
        inpstr = self.choose_files(pulls)
        allidxs = self.parse_input_string(inpstr, len(pulls))
        for pidx in allidxs:
            self.trash_a_file(pulls[pidx])

# ## private methods ##
    def trash_a_file(self, pfile, pname=None):
        lg.debug("title %s id %s" % (pfile['title'], pfile['id']))
        try:
            self.service.files().trash(fileId=pfile['id']).execute()
        except errors.HttpError, error:
            lg.error('An error occured: %s' % error)
            return None
        self.info("file %s is trashed" % pfile['title'])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import argparse
import re
import logging
import colorama
import ConfigParser

from gdrv import global_mod as gm
from gdrv.ftp import DriveFtp
from gdrv.commands import *


reload(sys)
sys.setdefaultencoding('utf-8')

lgr = logging.getLogger()
lgr.setLevel(logging.ERROR)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(name)s] %(levelname)s - %(message)s')
ch.setFormatter(formatter)
lgr.addHandler(ch)

lg = logging.getLogger("DRIVE")
lg.setLevel(logging.ERROR)

drive_commands = {}


def set_logging_level(plevel=0):
    global lg
    lvls = [logging.CRITICAL,
            logging.ERROR,
            logging.WARNING,
            logging.INFO,
            logging.DEBUG]
    # print min(plevel, len(lvls) - 1)
    lg.setLevel(lvls[min(plevel, len(lvls) - 1)])


def load_default_config():
    gm.config = ConfigParser.RawConfigParser()
    gm.config.add_section('api')
    gm.config.set('api', 'client_id',
                  '741789474926.apps.googleusercontent.com')
    gm.config.set('api', 'client_secret', 'ZZI0GSvLNLEHYUNsBsrmkG2y')
    gm.config.set('api', 'redirect_url', 'http://127.0.0.1')
    gm.config.set('api', 'scope', 'https://www.googleapis.com/auth/drive')
    gm.config.set('api', 'storage', '~/.config/gdrv/credentials.dat')


def get_config(args=None):
    load_default_config()

    # ## YMK TODO: -f filename and -w filename
    lg.debug("YMK dump config api ")
    lg.debug(gm.config.items('api'))
    last_cfg_name = ""
    for cfg_path in gm.config_paths:
        cfg_name = os.path.expanduser(cfg_path + '.' + gm.app_name + 'rc')
        # lg.debug("YMK config file check %s", cfg_name)
        if os.path.exists(cfg_name):
            last_cfg_name = cfg_name
            gm.config.read(cfg_name)

    if args.write_config is True:
        if last_cfg_name == "":
            new_cfg_name = os.path.expanduser(gm.config_paths[0] + '.' +
                                              gm.app_name + 'rc')
            lg.debug("YMK let's make a new one %s", new_cfg_name)
            with open(new_cfg_name, 'w+') as configfile:
                gm.config.write(configfile)


def main():
    global drive_commands

    parser = argparse.ArgumentParser(
        description='YMK google drive command line tool')
    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help='increse verbosity/logging level')
    parser.add_argument('-w', '--write-config', action='store_true',
                        help='write a default config')
    parser.add_argument('-V', '--version', action='version',
                        version="%s" % gm.version,
                        help='show version infomation')
    subparser = parser.add_subparsers(help='drive sub command',
                                      dest='command_name')

    subparser.add_parser('ftp', help='interactive mode like sftp, lftp')

    # ## YMK TODO: should use factory pattern
    drive_command_classes = {'list': CommandList,
                             'push': CommandPush,
                             'pull': CommandPull,
                             'mkdir': CommandMkdir,
                             'search': CommandSearch,
                             'trash': CommandTrash,
                             'url': CommandUrl,
                             'share': CommandShare,
                             'init': CommandInit}
    map(lambda (k, ac): ac.static_add_sub_command_parser(subparser),
        drive_command_classes.iteritems())

    args = parser.parse_args()
    set_logging_level(args.verbose)

    get_config(args)

    drive_commands = dict((val, drive_command_classes[val](gm.config))
                          for (key, val) in enumerate(drive_command_classes))

    colorama.init(autoreset=False)
    if args.command_name == 'ftp':
        DriveFtp(drive_commands, gm.config, parser).cmdloop()
    else:
        drive_commands[args.command_name](args)
    colorama.deinit()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import fnmatch
import logging
from apiclient import errors
from command_base import DriveServiceCommand
from command_list import CommandList

lg = logging.getLogger("DRIVE.URL")
# lg.setLevel(logging.INFO)


class CommandUrl(CommandList):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('url',
                                        help='command url help')
        cmdparser.add_argument('src', nargs='+',
                               help='patterns to list in google drive')

        # cmdparser.add_argument('-a', '--altlink', action='store_true',
        #                       help='show alternateLink to redirect google drive file information page')

    def do_service_command(self):
        """url files
        """

        lg.debug("YMK in do_command")
        lg.debug(self.args)

        files = self.get_all_src_files(self.args.src, False)

        if len(files) == 0:
            sys.exit("No files matched in drive")

        self.url_files(files)

# ## private methods ##
    def url_files(self, files):
        for pidx in range(len(files)):
            perms = self.permission_list(files[pidx]['id'])
            perms = filter(lambda p: p['type'] == 'anyone', perms)
            shared = 'shared' if len(perms) > 0 else ''
            # lg.debug("shared ? %s" % shared)
            # for aperm in perms:
            #     lg.debug("a perm kind %s type %s role %s" % (aperm['kind'], aperm['type'], aperm['role']))

            # TODO list display
            if 'webContentLink' in files[pidx]:
                link = files[pidx]['webContentLink']
                # self.info("%d %s wcl %s" % (pidx, files[pidx]['title'], files[pidx]['webContentLink']))
            elif 'alternateLink' in files[pidx]:
                link = files[pidx]['alternateLink']
                # self.info("%d %s atl %s" % (pidx, files[pidx]['title'], files[pidx]['alternateLink']))
            else:
                link = files[pidx]['id']
                # self.info("%d %s id %s" % (pidx, files[pidx]['title'], files[pidx]['id']))

            self.info("%2d %s \n  (%s)  %s" % (pidx, files[pidx]['title'], shared, link))




#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

__all__ = []

import pkgutil
import inspect

for loader, name, is_pkg in pkgutil.walk_packages(__path__):
    # print("YMK name {0}".format(name))
    module = loader.find_module(name).load_module(name)

    for name, value in inspect.getmembers(module):
        if name.startswith('__'):
            continue

        globals()[name] = value
        __all__.append(name)
# print("YMK __all__ {0}".format(__all__))



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import logging
import fnmatch
import httplib2
from apiclient import errors
from apiclient.discovery import build
from oauth2client.file import Storage

lg = logging.getLogger("BASE")
# lg.setLevel(logging.INFO)


class DriveCommand():
    """ A Drive Command Class """

    def __init__(self, pconfig):
        self.config = pconfig
        self.msgout = sys.stdout

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        pass

    def __call__(self, args=None):
        if args is not None:
            self.args = args
        self.do_drive_command()

    def do_drive_command(self):
        pass

# ## base command methods ##
    def info(self, *args):
        try:
            self.msgout.write(*args)
            self.msgout.write('\n')
            self.msgout.flush()
        except UnicodeError:
            pass

    def info_append(self, *args):
        try:
            self.msgout.write(*args)
            self.msgout.flush()
            # self.msgout.write('\n')
        except UnicodeError:
            pass

    def parse_input_string(self, pinstr, pmaxlen):
        idxs = []
        if pinstr == 'a':
            return range(pmaxlen)
        for acom in pinstr.split(','):
            arange = acom.split('-')
            # lg.debug("aidx ")
            # lg.debug(arange)
            try:
                if len(arange) == 1:
                    aidx = int(arange[0])
                    idxs.append(aidx)
                elif len(arange) == 2:
                    aidx = int(arange[0])
                    bidx = int(arange[1])
                    idxs.extend(range(aidx, bidx + 1))
            except ValueError:
                pass
            # lg.debug("aidx %d bidx %d") % (aidx, bidx)
        # ridx = filter(lambda x: x < pmaxlen, idxs)
        # lg.debug(ridx)
        return set(filter(lambda x: x < pmaxlen, idxs))


class DriveServiceCommand(DriveCommand):
    """ A Drive Service Command Class """

    def get_storage(self):
        self.storage = Storage(
            os.path.expanduser(self.config.get('api', 'storage')))

    def get_credentials(self):
        self.credentials = None
        self.get_storage()
        self.credentials = self.storage.get()

    def get_service(self):
        self.service = None
        self.get_credentials()
        if self.credentials is None or self.credentials.invalid:
            print "Please init oauth2 flow first"
        else:
            http = httplib2.Http()
            http = self.credentials.authorize(http)
            self.service = build('drive', 'v2', http=http)

    def do_drive_command(self):
        self.get_service()
        if self.service is not None:
            self.do_service_command()

    def do_service_command(self):
        pass

# ## helper drive apis ##
    def find_drive_files(self, psrcdir, pname,
                         hidedir=False, hidetrashed=True):
        matches = []
        files = self.get_all_children(psrcdir,
                                      hidedir=hidedir, hidetrashed=hidetrashed)
        for afile in files:
            if fnmatch.fnmatch(afile['title'], pname):
                matches.append(afile)
        return matches

    def get_all_children(self, psrcdir, hidedir=False, hidetrashed=True):
        parentid = self.find_parent_id(psrcdir)
        if parentid is None:
            lg.error("Can't find directory %s in drive" % psrcdir)
            sys.exit("Can't find directory %s in drive" % psrcdir)
        query = "'%s' in parents" % parentid
        if hidedir is True:
            query += " and mimeType != 'application/vnd.google-apps.folder'"
        if hidetrashed is True:
            query += " and trashed = false"
        return self.file_list(query)

    def find_parent_id(self, pdir, pmkdir=False):
        dirs = pdir.split('/')
        parentid = 'root'
        # for aidx in range(len(dirs)):
        for adir in dirs:
            # lg.debug("dirs %s" % (adir))
            if adir == '':
                continue
            children_dirs = self.check_children_dirs(adir, parentid)
            dirs_nums = len(children_dirs)
            if dirs_nums == 0:
                lg.error("Can't find directory %s" % (adir))
                return None
            elif dirs_nums > 1:
                lg.warn("Find %d instances of directory %s" % (
                    dirs_nums, adir))
            parentid = children_dirs[0]['id']
        return parentid

    def check_children_dirs(self, dirname, parent="root"):
        query = "mimeType = 'application/vnd.google-apps.folder'"
        query += " and title = '%s'" % dirname
        query += " and '%s' in parents" % parent
        # lg.debug("query %s" % query)
        children_dirs = self.file_list(query)
        # for adir in children_dirs:
        #     lg.debug("children %s id %s" % (adir['title'], adir['id']))
        return children_dirs

# ## basic drive apis ##
    def file_list(self, query=""):
        """Retrieve a list of File resources.

        Args:
          service: Drive API service instance.
        Returns:
          List of File resources.
        """
        # lg.debug("file_list query %s" % query)
        result = []
        page_token = None
        while True:
            try:
                param = {}
                if query != "":
                    param['q'] = query
                if page_token:
                    param['pageToken'] = page_token
                files = self.service.files().list(**param).execute()

                result.extend(files['items'])
                page_token = files.get('nextPageToken')
                if not page_token:
                    break
            except errors.HttpError, error:
                print 'An error occurred: %s' % error
                break
        return result

    def permission_list(self, pfile="root"):
        """Retrieve a list of permissions of the file

        Args:
          pfile: drive file id
        Returns:
          list of file permissions
        """
        # lg.debug("permission_list query %s" % query)
        result = []
        page_token = None
        while True:
            try:
                param = {}
                if page_token:
                    param['pageToken'] = page_token
                perms = self.service.permissions().list(fileId=pfile).execute()

                result.extend(perms['items'])
                page_token = perms.get('nextPageToken')
                if not page_token:
                    break
            except errors.HttpError, error:
                print 'An error occurred: %s' % error
                break
        return result

    # deprecated
    def children_list(self, parent="root", query=""):
        """Retrieve a list of File resources.

        Args:
          parent: parent id or alias 'root'
          query: query string
        Returns:
          List of File resources.
        """
        result = []
        page_token = None
        while True:
            try:
                param = {}
                if query != "":
                    param['q'] = query
                if page_token:
                    param['pageToken'] = page_token
                files = self.service.children().list(
                    folderId=parent, **param).execute()

                result.extend(files['items'])
                page_token = files.get('nextPageToken')
                if not page_token:
                    break
            except errors.HttpError, error:
                print 'An error occurred: %s' % error
                break
        return result



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

from setuptools import setup
from gdrv import global_mod as gm

setup(name='gdrv',
      version=gm.version,
      description='Another google drive command line interface program',
      url='https://github.com/yumaokao/gdrv',
      author='yumaokao',
      author_email='ymkq9h@gmail.com',
      license='MIT',
      packages=['gdrv', 'gdrv.commands'],
      entry_points={
          'console_scripts':
              ['gdrv = gdrv.main:main']
      },
      install_requires=[
          'google-api-python-client',
          'progressbar',
          'colorama'
      ],
      zip_safe=False)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

from os import getcwd, listdir, chdir
from os.path import isabs, isdir, isfile, join, split, normpath, basename
from colorama import Fore, Style
from cmd import Cmd


def real_do_gdrv(line):
    print("YMK do_gdrv")


def real_complete_gdrv(text, line, begidx, endidx):
    return ["yumaokao", "yumao.kao", "yumaokao74", "ymk74"]


class DriveFtp(Cmd):
    """ Google drive interactive mode """

    prompt = 'gdrv> '

    def __init__(self, commands, config, parser):
        self.commands = commands
        self.config = config
        self.parser = parser
        self.pwd = '/'
        self.parentid = 'root'
        # YMK: local should be fast enough
        # self.cache_lpwd = getcwd()
        # self.cache_files = None
        # self.cache_dirs = None
        Cmd.__init__(self)
        # YMK: don't think this is a good idea, though it's cool.
#        setattr(self, 'do_gdrv', real_do_gdrv)
#        setattr(self, 'complete_gdrv', real_complete_gdrv)

    def get_names(self):
        return dir(self)

    def do_exit(self, line):
        return True

    def do_EOF(self, line):
        return True

#   ### init ###
    def do_init(self, line):
        args = self.parser.parse_args()
        self.commands['init'](args)

# ##################
#   ### TRANSFOR ###
# ##################
#   ### pull ###
    def do_pull(self, line):
        # print(Fore.BLUE + "GDRV PULL: {0}".format(line) + Style.RESET_ALL)
        inodes = filter(lambda i:
                        i['title'] == line and
                        i['mimeType'] != 'application/vnd.google-apps.folder',
                        self.cache_inodes)
        if len(inodes) == 1:
            # print("url {0}".format(inodes[0]))
            self.commands['pull'].get_service()
            self.commands['pull'].pull_a_file(inodes[0])
        else:
            print("No such file or directory")

    def complete_pull(self, text, line, begidx, endidx):
        return filter(lambda i: i.startswith(text), self.cache_files)

#   ### push ###
    def do_push(self, line):
        # print(Fore.BLUE + "GDRV PUSH: {0}".format(line) + Style.RESET_ALL)
        lfile = join(getcwd(), line)
        if isfile(lfile):
            self.commands['push'].get_service()
            self.commands['push'].file_insert(lfile, basename(lfile), self.parentid)
        else:
            print("No such file or directory")

    def complete_push(self, text, line, begidx, endidx):
        cwd = getcwd()
        bname = text
        if text.endswith('..'):
            return [text + '/']
        if len(line.split()) > 1:
            (dname, bname) = split(line.split()[1])
            newdir = join(getcwd(), dname)
            # print("newdir {0}".format(newdir))
            if isdir(newdir):
                cwd = newdir
        inodes = filter(lambda i: i.startswith(bname), listdir(cwd))
        return inodes

# ################
#   ### REMOTE ###
# ################
#   ### pwd ###
    def do_pwd(self, line):
        print(Fore.BLUE + "GDRV PWD: {0}".format(self.pwd) + Style.RESET_ALL)

#   ### ls ###
    def do_ls(self, line):
        # print("YMK ls in {0} parentid {1}".format(self.pwd, self.parentid))
        # args = self.parser.parse_args(("list {0}".format(self.pwd)).split())
        # print("YMK ls args {0}".format(args))
        self.commands['list'].get_service()
        self.cache_inodes = self.commands['list'].get_all_children(self.pwd)
        self.cache_dirs = filter(lambda i:
                                 i['mimeType'] == 'application/vnd.google-apps.folder',
                                 self.cache_inodes)
        self.cache_dirs = map(lambda d: d['title'], self.cache_dirs)
        self.cache_files = filter(lambda i:
                                  i['mimeType'] != 'application/vnd.google-apps.folder',
                                  self.cache_inodes)
        self.cache_files = map(lambda d: d['title'], self.cache_files)
        # print("YMK ls cache inodes {0}".format(self.cache_inodes))
        # print("YMK ls cache dirs {0}".format(self.cache_dirs))
        # self.commands['list'](args)
        for d in sorted(self.cache_dirs):
            print(Fore.BLUE + d + Style.RESET_ALL)
        for f in sorted(self.cache_files):
            print(f)

#   ### cd ###
    def do_cd(self, line):
        # print("YMK ls in {0}".format(self.pwd))
        self.commands['list'].get_service()
        newpwd = line if isabs(line) else normpath(join(self.pwd, line))
        dirn = newpwd if newpwd.endswith('/') else newpwd + '/'
        parentid = self.commands['list'].find_parent_id(dirn)
        if parentid is not None:
            self.pwd = newpwd
            self.parentid = parentid

    def complete_cd(self, text, line, begidx, endidx):
        if text.endswith('..'):
            return [text + '/']
        return filter(lambda i: i.startswith(text), self.cache_dirs)

#   ### mkdir ###
    def do_mkdir(self, line):
        # print("YMK ls in {0}".format(self.pwd))
        self.commands['mkdir'].get_service()
        ndir = self.commands['mkdir'].dir_insert(line, self.parentid)
        if ndir is None:
            print("Could not new directory '{0}'", line)

    def complete_mkdir(self, text, line, begidx, endidx):
        return (filter(lambda i: i.startswith(text), self.cache_dirs)
                + filter(lambda i: i.startswith(text), self.cache_files))

#   ### trash ###
    def do_trash(self, line):
        # print(Fore.BLUE + "GDRV TRASH: {0}".format(line) + Style.RESET_ALL)
        inodes = filter(lambda i: i['title'] == line, self.cache_inodes)
        if len(inodes) == 1:
            # print("url {0}".format(inodes[0]))
            self.commands['trash'].get_service()
            self.commands['trash'].trash_a_file(inodes[0])
        else:
            print("No such file or directory")

    def complete_trash(self, text, line, begidx, endidx):
        return (filter(lambda i: i.startswith(text), self.cache_dirs)
                + filter(lambda i: i.startswith(text), self.cache_files))

#   ### url ###
    def do_url(self, line):
        self.commands['url'].get_service()
        pwd = self.pwd + '/' if not self.pwd.endswith('/') else self.pwd
        pline = pwd if line == "" else line
        if pline.endswith('/'):
            inodes = self.commands['url'].get_all_src_files(pline.split(), False)
        else:
            inodes = filter(lambda i: i['title'] == pline, self.cache_inodes)

        if len(inodes) == 0:
            print("No files matched in drive")
        else:
            self.commands['url'].url_files(inodes)

    def complete_url(self, text, line, begidx, endidx):
        return (filter(lambda i: i.startswith(text), self.cache_dirs)
                + filter(lambda i: i.startswith(text), self.cache_files))

#   ### share ###
    def do_share(self, line):
        # pline = self.pwd if line == "" else line
        # pline = pline if pline.endswith('/') else pline + '/'
        self.commands['share'].get_service()
        files = filter(lambda i: i['title'] == line, self.cache_inodes)
        # files = self.commands['share'].get_all_src_files(pline.split(), False)
        if len(files) == 0:
            print("No files matched in drive")
        else:
            for afile in files:
                perms = self.commands['share'].permission_list(afile['id'])
                perms = filter(lambda p: p['type'] == 'anyone', perms)
                shared = 'shared' if len(perms) > 0 else ''
                # print(" file {0} id {1} share [{2}]".format(afile['title'], afile['id'], shared))
                if not shared:
                    self.commands['share'].share_a_file(afile['id'])
                    print("shared {0}".format(afile['title']))
                else:
                    self.commands['share'].unshare_a_file(afile['id'])
                    print("unshared {0}".format(afile['title']))

    def complete_share(self, text, line, begidx, endidx):
        return (filter(lambda i: i.startswith(text), self.cache_dirs)
                + filter(lambda i: i.startswith(text), self.cache_files))

# ###############
#   ### LOCAL ###
# ###############
#   ### lpwd ###
    def do_lpwd(self, line):
        print(Fore.BLUE + "CWD: {0}".format(getcwd()) + Style.RESET_ALL)

#   ### lls ###
    def do_lls(self, line):
        inodes = listdir(getcwd())
        dirs = filter(lambda i: isdir(i), inodes)
        files = filter(lambda i: isfile(i), inodes)

        dirs = filter(lambda d: not d.startswith('.'), dirs)
        files = filter(lambda d: not d.startswith('.'), files)
        for d in dirs:
            print(Fore.BLUE + d + Style.RESET_ALL)
        for f in files:
            print(f)
        # print("{0}".format(files))

#   ### lcd ###
    def do_lcd(self, line):
        if isdir(line):
            chdir(line)
        self.cache_lpwd = getcwd()
        self.do_lpwd(None)

    def complete_lcd(self, text, line, begidx, endidx):
        # YMK cmd can't hanle '/' well with text ?
        cwd = getcwd()
        bname = text
        if text.endswith('..'):
            return [text + '/']
        if len(line.split()) > 1:
            (dname, bname) = split(line.split()[1])
            newdir = join(getcwd(), dname)
            # print("newdir {0}".format(newdir))
            if isdir(newdir):
                cwd = newdir
        # print("cwd {0}".format(cwd))
        # print("listdir {0}".format(listdir(cwd)))
        dirs = filter(lambda i: isdir(join(cwd, i)), listdir(cwd))
        dirs = filter(lambda i: i.startswith(bname), dirs)
        return dirs

# if __name__ == '__main__':
#    DriveFtp().cmdloop()



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import logging
import urllib2
import fnmatch
import colorama
import progressbar
from apiclient import errors
# from command_base import DriveServiceCommand
from command_list import CommandList

lg = logging.getLogger("DRIVE.PULL")
# lg.setLevel(logging.INFO)


class CommandPull(CommandList):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('pull',
                                        help='command pull help')
        cmdparser.add_argument('src', nargs='+',
                               help='google drive files')
        cmdparser.add_argument('-o', '--output', nargs=1,
                               help='desination')

    def do_service_command(self):
        """pull files
        """

        self.msgout = sys.stdout
        if self.args.output is not None:
            if self.args.output[0] == '-':
                self.msgout = sys.stderr

        # lg.debug(self.args)
        pulls = self.get_all_src_files(self.args.src, hidedir=True)

        if len(pulls) == 0:
            sys.exit("No files matched in drive")
        self.info(colorama.Fore.RED +
                  "Would you like to pull these files ?" +
                  colorama.Style.RESET_ALL)
        self.show_files_info(pulls, pnum=True)
        inpstr = self.choose_files(pulls)
        allidxs = self.parse_input_string(inpstr, len(pulls))
        if self.args.output is not None:
            if not len(allidxs) == 1:
                lg.error("Mutilple output filenames not yet supported")
            else:
                for pidx in allidxs:
                    self.pull_a_file(pulls[pidx], self.args.output[0])
        else:
            for pidx in allidxs:
                self.pull_a_file(pulls[pidx])

# ## private methods ##
    def pull_a_file(self, pfile, pname=None):
        # lg.debug("title %s url %s" % (pfile['title'], pfile['downloadUrl']))
        try:
            auth = {}
            self.credentials.apply(auth)
            # lg.debug("auth header %s" % auth)
            url = pfile.get('downloadUrl')
            if url is None:
                url = pfile['exportLinks']['application/pdf']

            req = urllib2.Request(url)
            for key, val in auth.iteritems():
                # lg.debug("auth header key %s val %s" % (key, val))
                req.add_header(key, val)
            res = urllib2.urlopen(req)
        except urllib2.HTTPError, e:
            lg.debug(e)
            sys.exit("%s: %s" % (pfile['title'], e))

        if res.info().getheader('Content-Length') is not None:
            http_size = int(res.info().getheader('Content-Length').strip())
        else:
            file_size = pfile.get('fileSize')
            http_size = 1024 * 1024 * 1024 if file_size is None else int(file_size)
        # drive_size = int(pfile['fileSize'])
        # lg.debug("size http %d drive %d" % (http_size, drive_size))

        self.info("%s downloading ..." % pfile['title'])
        if pname is None:
            tmpfile = "%s.part" % pfile['title']
        else:
            tmpfile = "%s.part" % pname
        size = http_size
        chunk_size = 1024 * 1024
        # with open(tmpfile, 'w') as fout:
        try:
            # lg.debug("[%s]" % pname)
            if pname is not None and pname == '-':
                fout = sys.stdout
            else:
                fout = open(tmpfile, 'w')
            pbar = progressbar.ProgressBar(
                widgets=[progressbar.Percentage(),
                         progressbar.Bar()],
                maxval=size).start()
            pull_size = 0
            buf = res.read(chunk_size)
            while buf != "":
                pull_size += len(buf)
                # lg.debug("%d of %d bytes downloaded" % (pull_size, http_size))
                pbar.update(pull_size)
                fout.write(buf)
                buf = res.read(chunk_size)
            pbar.finish()
        except IOError, e:
            if fout == sys.stdout:
                pass
            else:
                lg.error("IOError, %s" % e)
        finally:
            if not fout == sys.stdout:
                fout.close()

        if pull_size != http_size and http_size != 1024 * 1024 * 1024:
            lg.warn("only %d of %d bytes downloaded, maybe incompleted" %
                    (pull_size, http_size))
        else:
            if fout != sys.stdout:
                os.rename(tmpfile, pfile['title'])
            # print "%s download completed" % pfile['title']



#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
import os
import sys
import logging
import progressbar
from apiclient import errors
from apiclient.http import MediaFileUpload
from command_base import DriveServiceCommand

lg = logging.getLogger("DRIVE.MKDIR")
# lg.setLevel(logging.INFO)


class CommandMkdir(DriveServiceCommand):
    """ A Drive Command Class """

    @staticmethod
    def static_add_sub_command_parser(psub_par):
        cmdparser = psub_par.add_parser('mkdir',
                                        help='command mkdir help')
        # ## for query string composing ###
        cmdparser.add_argument('dst', nargs=1,
                               help='desination')
        cmdparser.add_argument('-p', '--prarents', nargs=1,
                               help='no error if existing,'
                               'make parent directories as needed')

    def do_service_command(self):
        """mkdir directory
        """

        lg.debug("YMK in do_command")
        lg.debug(self.args)
        # sys.stderr.write("YMK STDERR\n")

        newdir = os.path.split(self.args.dst[0])
        if newdir[1] == "":
            newdir = os.path.split(newdir[0])

        if newdir[1] == "":
            lg.error("Can't find directory %s" % newdir[1])
            sys.exit("Can't find directory %s" % newdir[1])

        parentid = self.find_dst_dir(newdir[0])
        if parentid is None:
            lg.error("Can't find directory %s in drive" % self.args.dst[0])
            sys.exit("Can't find directory %s in drive" % self.args.dst[0])

        dirid = self.dir_insert(newdir[1], parentid)

# ## private methods ##
    def dir_insert(self, dirname, parent_id):
        """Insert new directory.

        Args:
            parent_id: Parent folder's ID.
            dirname: name of the directory to insert.
        Returns:
            Inserted directory metadata if successful, None otherwise.
        """

        body = {
            'title': dirname,
            'mimeType': 'application/vnd.google-apps.folder',
            }
        # Set the parent folder.
        if parent_id:
            body['parents'] = [{'id': parent_id}]

        try:
            ndir = self.service.files().insert(
                body=body).execute()
            return ndir
        except errors.HttpError, error:
            lg.error('An error occured: %s' % error)
            return None

    def find_dst_dir(self, dstdir=None):
        if dstdir is None:
            dstdir = os.path.dirname(self.args.dst[0])
        return self.find_parent_id(dstdir)
